@@ -20,9 +20,121 @@ use Text::ParseWords qw(shellwords);
 
 sub DEBUG() { $config->{debug}->{ly} }
 
+sub ly2svg {
+    my ( $s, $pw, $elt ) = @_;
+
+    state $imgcnt = 0;
+    state $td = File::Temp::tempdir( CLEANUP => !$config->{debug}->{ly} );
+
+    $imgcnt++;
+    my $src  = File::Spec->catfile( $td, "tmp${imgcnt}.ly" );
+    my $svg  = File::Spec->catfile( $td, "tmp${imgcnt}.svg" );
+
+    my $fd;
+    unless ( open( $fd, '>:utf8', $src ) ) {
+	warn("Error in Lilypond embedding: $src: $!\n");
+	return;
+    }
+
+    my $need_version = 1;
+    my @pre;
+    for ( keys(%{$elt->{opts}}) ) {
+
+	if ( $_ eq "version" ) {
+	    push( @pre, "\\version \"", $elt->{opts}->{$_}, "\"" );
+	    warn ( "\\version \"", $elt->{opts}->{$_}, "\"\n" ) if DEBUG;
+	    $need_version = 0;
+	}
+	else {
+	    push( @pre, '%%'.$_." ".$elt->{opts}->{$_} );
+	    warn('%%'.$_." ".$elt->{opts}->{$_}."\n") if DEBUG;
+	}
+    }
+
+    for ( @{ $config->{delegates}->{ly}->{preamble} } ) {
+	push( @pre, $_ );
+	warn( "$_\n") if DEBUG;
+	$need_version = 0 if /^\\version\s+/;
+    }
+
+    if ( $need_version ) {
+	my $v = "2.21.0";
+	unshift( @pre, "\\version \"$v\"" );
+	warn("ly: no \\version seen, assuming \"$v\"\n");
+    }
+    printf $fd "$_\n" for @pre,
+      "#(ly:set-option 'crop #t)",
+      "\\header { tagline = ##f }";
+
+    @pre = ();
+    my @data = @{$elt->{data}};
+    while ( @data ) {
+	$_ = shift(@data);
+	unshift( @data, $_ ), last if /^[%\\]/; # LP data
+	push( @pre, $_ );
+    }
+    if ( @pre && !@data ) {	# no LP found
+	@data = @pre;
+	@pre = ();
+    }
+
+    my $kv = { %$elt };
+    $kv = parse_kv( @pre ) if @pre;
+    # Copy. We assume the user knows how to write LilyPond.
+    for ( @data ) {
+	print $fd $_, "\n";
+	warn($_, "\n") if DEBUG;
+    }
+
+    unless ( close($fd) ) {
+	warn("Error in Lilypond embedding: $src: $!\n");
+	return;
+    }
+
+    if ( $kv->{width} ) {
+	$pw = $kv->{width};
+    }
+
+    state $lilypond = findexe( "lilypond", "silent" );
+    unless ( $lilypond ) {
+	warn("Error in Lilypond embedding: missing 'lilypond' tool.\n");
+	return;
+    }
+
+    my @cmd = ( $lilypond, qw( -dno-point-and-click --svg ) );
+    push( @cmd, "--silent" ) unless DEBUG;
+    ( my $im1 = $svg ) =~ s/\.\w+$//;
+    push( @cmd, "-o", $im1, $src );
+    my $ret = sys( @cmd );
+
+    if ( $ret ) {
+	warn( sprintf( "Error in Lilypond embedding (ret = 0x%x)\n", $ret ) );
+	return;
+    }
+    if ( ! -s "$im1.cropped.svg" ) {
+	warn("Error in Lilypond embedding (no output?)\n");
+	return;
+    }
+
+    warn("SVG: ", -s $svg, " bytes\n");
+    my @res;
+    push( @res,
+	  { type => "svg",
+	    uri  => "$im1.cropped.svg",
+	    opts => { center => $kv->{center},
+		      scale  => $kv->{scale},
+		      split  => $kv->{split},
+		    } } );
+
+    return \@res;
+
+}
+
 sub ly2image {
     my ( $s, $pw, $elt ) = @_;
 
+    croak("Lilypond: Please adjust your delegate config to use handler \"ly2svg\" instead of \"ly2image\"");
+
     state $imgcnt = 0;
     state $td = File::Temp::tempdir( CLEANUP => !$config->{debug}->{ly} );
 
@@ -94,7 +206,7 @@ sub ly2image {
 	$pw = $kv->{width};
     }
 
-    state $lilypond = findexe("lilypond");
+    state $lilypond = findexe( "lilypond", "silent" );
     unless ( $lilypond ) {
 	warn("Error in Lilypond embedding: missing 'lilypond' tool.\n");
 	return;
@@ -122,7 +234,7 @@ sub ly2image {
 	  if $config->{debug}->{images} || DEBUG;
     }
     elsif ( is_msw() ) {
-	state $magick = findexe("magick");
+	state $magick = findexe( "magick", "silent" );
 	unless ( $magick ) {
 	    warn("Error in Lilypond embedding: missing 'imagemagick/convert' tool.\n");
 	    return;
@@ -130,7 +242,7 @@ sub ly2image {
 	@cmd = ( $magick, "convert" );
     }
     else {
-	state $convert = findexe("convert");
+	state $convert = findexe( "convert", "silent" );
 	unless ( $convert ) {
 	    warn("Error in Lilypond embedding: missing 'imagemagick/convert' tool.\n");
 	    return;
