@@ -14,14 +14,15 @@ binmode STDERR => ':utf8';
 
 package ChordPro::Testing;
 
-use parent 'Exporter';
+our $VERSION = "6.000";
+
+use base 'Exporter';
 our @EXPORT = qw( $config );
 
 use Test::More ();
 
-use ChordPro::Files;
+use App::Packager ( ':name', 'ChordPro' );
 use ChordPro::Config;
-use ChordPro::Paths;
 use ChordPro::Chords;
 
 sub import {
@@ -31,6 +32,7 @@ sub import {
     -d "t" && chdir "t";
 
     $::running_under_test = 1;
+    App::Packager->export_to_level(1);
     Test::More->export_to_level(1);
     $pkg->export_to_level( 1, undef, @EXPORT );
 }
@@ -41,7 +43,7 @@ sub is_deeply {
     if ( ref($got) eq 'HASH' && ref($expect) eq 'HASH' ) {
 	fixchords($got) if $got->{body};
 
-	for ( qw( config generate ) ) {
+	for ( qw( config ) ) {
 	    delete $got->{$_} unless exists $expect->{$_};
 	}
 	if ( $got->{chordsinfo} ) {
@@ -54,9 +56,7 @@ sub is_deeply {
 		}
 	    }
 	}
-	for ( qw( instrument user key_from key_actual chords numchords
-		  _configversion bookmark
-	       ) ) {
+	for ( qw( instrument user key_from key_actual chords numchords ) ) {
 	    delete $got->{meta}->{$_} unless exists $expect->{meta}->{$_};
 	}
     }
@@ -82,7 +82,7 @@ ChordPro::Chords::add_config_chord
 no warnings 'redefine';
 
 sub getresource {
-    CP->findres($_[0]);
+    App::Packager::U_GetResource(@_);
 }
 }
 
@@ -110,13 +110,15 @@ sub cmp {
     }
 }
 
+use File::LoadLines qw( loadlines );
+
 sub differ {
     my ($file1, $file2) = @_;
     $file2 = "$file1" unless $file2;
     $file1 = "$file1";
 
-    my @lines1 = fs_load( $file1, { fail => 'hard' } );
-    my @lines2 = fs_load( $file2, { fail => 'hard' } );
+    my @lines1 = loadlines($file1);
+    my @lines2 = loadlines($file2);
     my $linesm = @lines1 > @lines2 ? @lines1 : @lines2;
     for ( my $line = 1; $line < $linesm; $line++ ) {
 	next if $lines1[$line] eq $lines2[$line];
