@@ -66,7 +66,7 @@ sub expand_tilde ( $dir ) {
 
 push( @EXPORT, 'expand_tilde' );
 
-sub findexe ( $prog ) {
+sub findexe ( $prog, $silent = 0 ) {
     my @path;
     if ( MSWIN ) {
 	$prog .= ".exe" unless $prog =~ /\.\w+$/;
@@ -84,7 +84,7 @@ sub findexe ( $prog ) {
 	}
     }
     warn("Could not find $prog in ",
-	 join(" ", map { qq{"$_"} } @path), "\n");
+	 join(" ", map { qq{"$_"} } @path), "\n") unless $silent;
     return;
 }
 
@@ -139,6 +139,11 @@ push( @EXPORT, 'make_preprocessor' );
 
 sub parse_kv ( @lines ) {
 
+    if ( is_macos() ) {
+	# MacOS has the nasty habit to smartify quotes.
+	@lines = map { s/“/"/g; s/”/"/g; s/‘/'/g; s/’/'/gr;} @lines;
+    }
+
     use Text::ParseWords qw(shellwords);
     my @words = shellwords(@lines);
 
@@ -258,4 +263,10 @@ sub maybe ( $key, $value, @rest ) {
 }
 push( @EXPORT, "maybe" );
 
+# Min/Max.
+sub min { $_[0] < $_[1] ? $_[0] : $_[1] }
+sub max { $_[0] > $_[1] ? $_[0] : $_[1] }
+
+push( @EXPORT, "min", "max" );
+
 1;
