@@ -7,7 +7,7 @@ use utf8;
 
 class  SVGPDF;
 
-our $VERSION = '0.091';
+our $VERSION = '0.070';
 
 =head1 NAME
 
@@ -46,24 +46,14 @@ with L<PDF::Builder>, L<PDF::API2> and compatible PDF packages.
 The main method is process(). It takes the SVG from an input source, see
 L</INPUT>.
 
-=head1 COORDINATES & UNITS
-
-SVG coordinates run from top-left to bottom-right.
-
-Dimensions without units are B<pixels>, at 96 pixels / inch. E.g.,
-C<width="96"> means 96px (pixels) and is equal to 72pt (points) or 1in (inch).
-
-For font sizes, CSS defines C<em> to be equal to the font size, and
-C<ex> is half of the font size.
-
-=head1 CONSTRUCTORS
-
-=head2 SVGPDF->new($pdf)
+=head1 CONSTRUCTOR
 
 In its most simple form, a new SVGPDF object can be created with a
 single argument, the PDF document.
 
-There are a few optional arguments, these can be specified as
+     $svg = SVGPDF->new($pdf);
+
+There are a few additional arguments, these must be specified as
 key/value pairs.
 
 =over 8
@@ -155,7 +145,6 @@ use SVGPDF::Element;
 use SVGPDF::CSS;
 use SVGPDF::FontManager;
 use SVGPDF::PAST;
-use SVGPDF::Colour;
 
 # The SVG elements.
 use SVGPDF::Circle;
@@ -206,8 +195,8 @@ sub BUILDARGS ( @args ) {
 }
 
 BUILD {
+    $verbose      = $atts->{verbose}      // 1;
     $debug        = $atts->{debug}        || 0;
-    $verbose      = $atts->{verbose}      // $debug;
     $grid         = $atts->{grid}         || 0;
     $prog         = $atts->{prog}         || 0;
     $debug_styles = $atts->{debug_styles} || $debug > 1;
@@ -241,27 +230,6 @@ The font size to be used for dimensions in 'ex' and 'em' units.
 
 This value overrides the value set in the constructor.
 
-=item combine
-
-An SVG can produce multiple XObjects, but sometimes these should be
-kept as a single image.
-
-There are two ways to combine the image objects. This can be selected
-by setting $opts{combine} to either C<"stacked"> or C<"bbox">.
-
-Type C<"stacked"> (default) stacks the images on top of each other,
-left sides aligned. The bounding box of each object is only used to
-obtain the width and height.
-
-Type C<"bbox"> stacks the images using the bounding box details. The
-origins of the images are vertically aligned and images may protrude
-other images when the image extends below the origin.
-
-=item sep
-
-When combining images, add additional vertical space between the
-individual images.
-
 =back
 
 =cut
@@ -292,13 +260,6 @@ method process ( $data, %options ) {
     # Restore.
     $fontsize = $save_fontsize;
 
-    my $combine = $options{combine} // "none";
-    if ( $combine ne "none" && @$xoforms > 1 ) {
-	my $sep = $options{sep} || 0;
-	$xoforms = $self->combine_svg( $xoforms,
-				       type => $combine, sep => $sep );
-    }
-
     # Return (hopefully a stack of XObjects).
     return $xoforms;
 }
@@ -473,7 +434,7 @@ method handle_svg ( $e ) {
 	    $self->_dbg("valign: ", $style->{'vertical-align'}, " ",
 			"\$vb[1] -> $vb[1]");
 	}
-	$vbox = sprintf("%.2f %.2f %.2f %.2f (inferred)", @vb);
+	$vbox = "@vb (inferred)";
     }
 
     $svg->nfi("disproportional vbox/width/height")
@@ -489,11 +450,9 @@ method handle_svg ( $e ) {
     $xo->bbox(@bb);
 
     if ( my $c = $style->{"background-color"} ) {
-	if ( $c ne "none" ) {
-	    $xo->fill_color($c);
-	    $xo->rectangle(@bb);
-	    $xo->fill;
-	}
+	$xo->fill_color($c);
+	$xo->rectangle(@bb);
+	$xo->fill;
     }
 
     # Set up result forms.
@@ -560,68 +519,6 @@ method handle_svg ( $e ) {
     $self->_dbg( "==== end ", $e->{name}, " ====" );
 }
 
-sub min( $a, $b ) { $a < $b ? $a : $b }
-sub max( $a, $b ) { $a > $b ? $a : $b }
-
-method combine_svg( $forms, %opts ) {
-    my $type = $opts{type} // "stacked";
-    return $forms if $type eq "none";
-
-    my ( $xmin, $ymin, $xmax, $ymax );
-    my $y = 0;
-    my $x = 0;
-    my $sep = $opts{sep} || 0;
-    my $nx;
-
-    if ( $type eq "bbox" ) {
-	warn("Combining ", scalar(@$forms), " XObjects\n")
-	  if $verbose;
-	...;
-	$nx->bbox( $xmin, $ymax, $xmax, 0 );
-    }
-    else {
-	my $i = 0;
-	for my $xo ( @$forms ) {
-	    $xo = $xo->{xo};
-	    my @bb = $xo->bbox;
-	    my $w = abs( $bb[2] - $bb[0] );
-	    my $h = abs( $bb[3] - $bb[1] );
-
-	    $nx //= $pdf->xo_form;
-	    my @xy = ( $x - min($bb[0],$bb[2]), $y - max($bb[1],$bb[3]) );
-	    warn(sprintf("Stack obj %2d: %.2f %.2f %.2f %.2f \@ %.2f %.2f\n",
-			 ++$i, @bb, @xy ) ) if $verbose;
-	    $nx->object( $xo, @xy );
-	    $y -= $h;
-
-	    if ( defined $xmax ) {
-		$xmax = $w if $w > $xmax;
-	    }
-	    else {
-		$xmax = $w;
-		$xmin = 0;
-	    }
-	    $ymax = $y;
-	    $y -= $sep;
-	}
-	warn("Stacked ", scalar(@$forms), " XObjects => bb",
-	     ( map { sprintf(" %.2f", $_ ) } $xmin, $ymax, $xmax, 0 ),
-	     "\n")
-	  if $verbose;
-	$nx->bbox( $xmin, $ymax, $xmax, 0 );
-    }
-
-
-    return [ { xo      => $nx,
-	       width   => $xmax - $xmin,
-	       height  => -$ymax,
-	       vwidth  => $xmax - $xmin,
-	       vheight => -$ymax,
-	       bbox    => [ $xmin, $ymax, $xmax, 0 ],
-	       vbox    => [ $xmin, -$ymax, $xmax-$xmin, $ymax ],
-	     } ];
-}
-
 ################ Service ################
 
 method vb2bb( @vb ) {
@@ -744,7 +641,7 @@ An SVG document on disk, specified as the name of the document.
 
 =item *
 
-A file handle, opened on a SVG document, specified as a glob
+A file handle, openened on a SVG document, specified as a glob
 reference. You can use C<\*DATA> to append the SVG data after a
 C<__DATA__> separator at the end of the program.
 
@@ -772,10 +669,6 @@ The viewBox as specified in the SVG element.
 If no viewBox is specified it is set to C<0 0> I<W H>, where I<W> and
 I<H> are the width and the height.
 
-=item C<bbox>
-
-Same as the C<vbox>, but using bottom-left and top-right coordinates.
-
 =item C<width>
 
 The width of the XObject, as specified in the SVG element or derived
@@ -816,43 +709,26 @@ match is found, it is resolved and the font is set. If there is no
 appropriate CSS rule for this font, the callback is called with the
 following arguments:
 
-    ( $self, %args )
+    ( $self, $pdf, $style )
 
-where C<%args> contains at least:
+where C<$pdf> is de PDF document and C<$style> a hash reference that
+contains values for C<font-family>, C<font-style>, C<font-weight>, and
+C<font-size>. Don't touch C<$self>, it is undocumented for a reason.
 
-=over
-
-=item pdf
-
-A reference to the current PDF document.
-
-=item style
-
-A hash reference that contains values for C<font-family>,
-C<font-style>, C<font-weight>, and C<font-size>.
-
-=back
-
-Do not touch C<$self>, it is undocumented for a reason.
-
-The callback function can use the contents of C<< $args->{style} >> to
-select an appropriate font and return it.
+The callback function can use the contents of C<$style> to select an
+appropriate font and return it.
 
 SVGPDF will try to call the font handler callback only once for each
 combination of family, style and weight. If the callback function
 returns a 'false' result SVGPDF will try other alternatives to find a
 font.
 
-Example of an (extremely simplified) callback.
-It yields the corefont Helvetica when the requested font-family is
-C<sans>, and Times-Roman for everything else. Styles and weights are
-ignored.
+Example of an (extremely simplified) callback:
 
     sub simple_font_handler {
-        my ( $self, %args ) = @_;
-        my $pdf = $args->{pdf};
+        my ( $self, $pdf, $style ) = @_;
 
-	my $family = $args->{style}->{'font-family'};
+	my $family = $style->{'font-family'};
 
 	my $font;
 	if ( $family eq 'sans' ) {
@@ -865,7 +741,7 @@ ignored.
         return $font;
     }
 
-If no callback function is set, SVGPDF will recognize the 14 standard
+If no callback function is set, SVGPDF will recognize the standard
 PDF corefonts, and aliases C<serif>, C<sans> and C<mono>.
 
 B<IMPORTANT: With the standard corefonts only characters of the
@@ -907,7 +783,7 @@ line,
 path,
 polygon,
 polyline,
-rect,
+rect (no rounded corners),
 text and tspan (no white-space styles).
 
 =item *
@@ -960,10 +836,6 @@ Shades, gradients, patterns and animations.
 
 =item *
 
-Shape rendering attributes.
-
-=item *
-
 Transparency.
 
 =item *
@@ -1019,7 +891,7 @@ L<https://github.com/sciurius/perl-SVGPDF/issues>.
 
 =head1 LICENSE
 
-Copyright (C) 2022,2024 Johan Vromans,
+Copyright (C) 2022.2023 Johan Vromans,
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided under the terms of the Simplified
